package com.hzya.frame.plugin.lets.dingtalk.push.serivce.impl;

import cn.hutool.core.lang.Assert;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.dingtalk.api.response.OapiV2UserGetbymobileResponse;
import com.hzya.frame.plugin.lets.dingtalk.push.serivce.PushU8CService;
import com.hzya.frame.plugin.lets.dingtalk.utils.DingTalkUtils;
import com.hzya.frame.plugin.lets.dingtalk.utils.FileUtil;
import com.hzya.frame.plugin.lets.dingtalk.vo.DjFileVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.DjzbVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.FileModuleVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.SysFileVO;
import com.hzya.frame.plugin.lets.plugin.adjust.AdjustInPluginInitializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

/**
 * u8c单据推送钉钉表单
 * fileBase64s为文件转了base64之后
 */
@Service("PushU8CService")
public class PushU8CServiceImpl implements PushU8CService {

    @Value("${DING.CorpId}")
    private String CorpId;
    @Value("${DING.APIToken}")
    private String APIToken;

    @Value("${DING.APPKEY}")
    private String APPKEY;
    @Value("${DING.AppSecret}")
    private String AppSecret;
    @Value("${DING.SPACEID}")
    private String SPACEID;

    @Value("${DING.APPROVE_CODE.SPECIAL_SERVICE_PROCESSING}")
    private String SPECIAL_SERVICE_PROCESSING;
    @Value("${DING.APPROVE_FILE_SPACE.SPECIAL_SERVICE_PROCESSING}")
    private String SPECIAL_SERVICE_PROCESSING_SPACE;

    @Value("${DING.APPROVE_CODE.PURCHASE_PAYMENT_REQUEST_NEW}")
    private String PURCHASE_PAYMENT_REQUEST_NEW;
    @Value("${DING.APPROVE_FILE_SPACE.PURCHASE_PAYMENT_REQUEST_NEW}")
    private String PURCHASE_PAYMENT_REQUEST_NEW_SPACE;

    @Value("${DING.APPROVE_CODE.PURCHASE_PAYMENT_REQUEST_SHOP}")
    private String PURCHASE_PAYMENT_REQUEST_SHOP;
    @Value("${DING.APPROVE_FILE_SPACE.PURCHASE_PAYMENT_REQUEST_SHOP}")
    private String PURCHASE_PAYMENT_REQUEST_SHOP_SPACE;


    Logger logger = LoggerFactory.getLogger(PushU8CServiceImpl.class);


    /** 委外采购（付款单）->采购付款申请(新) */
    @Override
    public String outSource(DjFileVO djFileVO) {
        //校验参数
        checkParameter(djFileVO);
        try{
            //获取钉钉参数
            GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);

            //制单人的钉钉unionid：通过手机号获取钉钉的userid在获取unionid   13783530043
            OapiV2UserGetbymobileResponse userByMobile = DingTalkUtils.getUserByMobile(tokenBody.getAccessToken(), "13783530043");
            String userid = userByMobile.getResult().getUserid();
            OapiV2UserGetResponse userById = DingTalkUtils.getUserById(tokenBody.getAccessToken(), userid);
            String unionid = userById.getResult().getUnionid();


            //base转文件流-上传钉盘-拿到钉盘信息。可能是List集合
            List<FileModuleVO> fileModuleVOList = baseTransformFile2uploadDing(djFileVO.getSysFileVOS(),tokenBody.getAccessToken(),unionid,SPECIAL_SERVICE_PROCESSING_SPACE,SPECIAL_SERVICE_PROCESSING_SPACE);
            System.out.println(fileModuleVOList);


        }catch (Exception e){
            e.printStackTrace();
            logger.error("委外采购（付款单）->采购付款申请（新）报错",e);
        }
        return "110";
    }

    /** 代理采购（付款单）->采购付款申请(店群专用) */
    @Override
    public String agency(DjFileVO djFileVO) {
        checkParameter(djFileVO);
        return null;
    }

    /** 调拨单内部交易（付款单）->特殊业务处理(新) */
    @Override
    public String transfer(DjFileVO djFileVO) {
        checkParameter(djFileVO);
        return null;
    }

    /**
     * 检查参数，VO不能为空，文件不能空
     * @param djFileVO:djzbVO+sysFileVOS
     */
    public void checkParameter(DjFileVO djFileVO){
        Assert.notNull(djFileVO.getDjzbVO(),"u8c单据推送钉钉表单,检查参数，VO不能为空");
        if (djFileVO.getSysFileVOS().size() == 0) {
            Assert.state(false,"u8c单据推送钉钉表单,检查参数，文件不能空");
        }
    }

    /**
     * 文件转换
     * 将base64转为文件流上传钉盘，并返回钉钉的 List<FileModuleVO>
     * @Param fileVOS 文件集合
     * @Param token token
     * @Param unionid unionid
     * @Param spaceId 空间信息
     * @Param parentSpaceId 父级空间id
     */
    public List<FileModuleVO> baseTransformFile2uploadDing(List<SysFileVO> fileVOS,String token,String unionid,String spaceId,String parentSpaceId) throws IOException {
        List<FileModuleVO> fileModuleVOList=new ArrayList<>();
        for (SysFileVO fileVO : fileVOS) {
            // 指定输出文件路径
            Path outputPath = Paths.get(fileVO.getFileName()+"."+fileVO.getType());
            // 1. 解码Base64字符串并写入文件
            byte[] decodedBytes = Base64.getDecoder().decode(fileVO.getBaseCode());
            Files.write(outputPath, decodedBytes);
            logger.info("文件已成功创建: " + outputPath);
            // 2. 将文件内容读取到ByteArrayOutputStream
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            try (InputStream inputStream = Files.newInputStream(outputPath)) {
                byte[] buffer = new byte[1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    byteArrayOutputStream.write(buffer, 0, bytesRead);
                }
            }catch(Exception e){
                e.printStackTrace();
            }
            // 3. 打印ByteArrayOutputStream的内容（可选）
            byte[] fileContent = byteArrayOutputStream.toByteArray();
            // 4. 文件流上传钉盘
            FileModuleVO dingDileModuleVO = FileUtil.get4DingDileModuleVO(token, unionid, spaceId, fileVO, byteArrayOutputStream, parentSpaceId);

            fileModuleVOList.add(dingDileModuleVO);
        }
        return fileModuleVOList;
    }

}
